/*
HOWTO: Converting Colors Between RGB and HLS (HBS)
ID: Q29240 

 

--------------------------------------------------------------------------------
The information in this article applies to:

Microsoft Windows Software Development Kit (SDK) 3.1 
Microsoft Win32 Application Programming Interface (API), used with:
Microsoft Windows NT, versions 3.5, 3.51, 4.0 
Microsoft Windows 95

--------------------------------------------------------------------------------


SUMMARY
The code fragment below converts colors between RGB (Red, Green, Blue) and
HLS/HBS (Hue, Lightness, Saturation/Hue, Brightness, Saturation). 



MORE INFORMATION
RGBtoHLS() takes a DWORD RGB value, translates it to HLS, and stores 
the results in the global vars H, L, and S. HLStoRGB takes the current
values of H, L, and S and returns the equivalent value in an RGB DWORD. 
The vars H, L, and S are only written to by: 

RGBtoHLS (initialization)


The scroll bar handlers


A point of reference for the algorithms is Foley and Van Dam, 
"Fundamentals of Interactive Computer Graphics," Pages 618-19. Their 
algorithm is in floating point. CHART implements a less general 
(hardwired ranges) integral algorithm. 

There are potential round-off errors throughout this sample. 
((0.5 + x)/y) without floating point is phrased ((x + (y/2))/y), 
yielding a very small round-off error. This makes many of the 
following divisions look strange. 
*/

// Add to PanPlot: 12.01.2001, Dr. Rainer Sieger, Uni Bremen

#include <windows.h>

#include "Struct.h"
#include "Globals.h"

#include "ColorConversion.h"

HLSColor RGBtoHLS(DWORD lRGBColor)
{
  WORD R,G,B;          /* input RGB values */ 
  HLSColor HLS;				 /* return HLS values */
  BYTE cMax,cMin;      /* max and min RGB values */ 
  WORD Rdelta,Gdelta,Bdelta; /* intermediate value: % of spread from max */
    
  /* get R, G, and B out of DWORD */ 
  R = GetRValue(lRGBColor);
  G = GetGValue(lRGBColor);
  B = GetBValue(lRGBColor);

  /* calculate lightness */ 
  cMax    = (unsigned char) (max( max(R,G), B));
  cMin    = (unsigned char) (min( min(R,G), B));
  HLS.lum = (unsigned char) ((((cMax+cMin)*HLSMAX) + RGBMAX )/(2*RGBMAX));

  if (cMax == cMin) {           /* r=g=b --> achromatic case */ 
      HLS.sat = 0;               /* saturation */ 
      HLS.hue = UNDEFINED;       /* hue */ 
  }
  else {                        /* chromatic case */ 
      /* saturation */ 
      if (HLS.lum <= (HLSMAX/2))
        HLS.sat = (unsigned char) (( ((cMax-cMin)*HLSMAX) + ((cMax+cMin)/2) ) / (cMax+cMin));
      else
        HLS.sat = (unsigned char) (( ((cMax-cMin)*HLSMAX) + ((2*RGBMAX-cMax-cMin)/2) ) / (2*RGBMAX-cMax-cMin));

      /* hue */ 
			Rdelta = (unsigned short) (( ((cMax-R)*(HLSMAX/6)) + ((cMax-cMin)/2) ) / (cMax-cMin));
			Gdelta = (unsigned short) (( ((cMax-G)*(HLSMAX/6)) + ((cMax-cMin)/2) ) / (cMax-cMin));
			Bdelta = (unsigned short) (( ((cMax-B)*(HLSMAX/6)) + ((cMax-cMin)/2) ) / (cMax-cMin));

      if (R == cMax)
        HLS.hue = (unsigned char) (Bdelta - Gdelta);
      else if (G == cMax)
        HLS.hue = (unsigned char) ((HLSMAX/3) + Rdelta - Bdelta);
      else /* B == cMax */ 
        HLS.hue = (unsigned char) (((2*HLSMAX)/3) + Gdelta - Rdelta);

      if (HLS.hue < 0)
        HLS.hue += HLSMAX;
      if (HLS.hue > HLSMAX)
        HLS.hue -= HLSMAX;
  }
  return(HLS);
}


// *********************************************************************
// *********************************************************************
// *********************************************************************
// *********************************************************************
   

DWORD HLStoRGB(WORD hue, WORD lum, WORD sat)
{
    WORD R,G,B;                /* RGB component values */ 
    WORD  Magic1,Magic2;       /* calculated magic numbers (really!) */ 

    if (sat == 0) {            /* achromatic case */ 
      R=G=B= (unsigned short) ((lum*RGBMAX)/HLSMAX);
      if (hue != UNDEFINED) {
        /* ERROR */ 
      }
    }
    else  {                    /* chromatic case */ 
      /* set up magic numbers */ 
      if (lum <= (HLSMAX/2))
        Magic2 = (unsigned char) ((lum*(HLSMAX + sat) + (HLSMAX/2))/HLSMAX);
      else
        Magic2 = (unsigned char) (lum + sat - ((lum*sat) + (HLSMAX/2))/HLSMAX);
      Magic1 = (unsigned char) (2*lum-Magic2);

      /* get RGB, change units from HLSMAX to RGBMAX */ 
      R = (unsigned short) ((HueToRGB(Magic1,Magic2,(unsigned short) (hue+(HLSMAX/3)))*RGBMAX + (HLSMAX/2))/HLSMAX);
      G = (unsigned short) ((HueToRGB(Magic1,Magic2,hue)*RGBMAX + (HLSMAX/2)) / HLSMAX);
      B = (unsigned short) ((HueToRGB(Magic1,Magic2,(unsigned short) (hue-(HLSMAX/3)))*RGBMAX + (HLSMAX/2))/HLSMAX);
  }

		return(RGB(R,G,B));
} 

// *********************************************************************
// *********************************************************************

/* utility routine for HLStoRGB */ 
WORD HueToRGB(WORD n1, WORD n2, WORD hue)
{
  /* range check: note values passed add/subtract thirds of range */ 
  if (hue < 0)
      hue += HLSMAX;

  if (hue > HLSMAX)
      hue -= HLSMAX;

  /* return r,g, or b value from this tridrant */ 
  if (hue < (HLSMAX/6))
      return(unsigned short) (( n1 + (((n2-n1)*hue+(HLSMAX/12))/(HLSMAX/6)) ));
  if (hue < (HLSMAX/2))
      return( n2 );
  if (hue < ((HLSMAX*2)/3))
      return(unsigned short) (( n1 +    (((n2-n1)*(((HLSMAX*2)/3)-hue)+(HLSMAX/12))/(HLSMAX/6)) ));
  else
      return(n1);
}
